# Copyright (C) 2004  Anthony Carrico

"""Fetch and manipulate RelaxNG schemas"""

from giftfile import funkxml
from giftfile.funkxml import Element, NsDecl, Attribute, Root
import libxml2

_relaxng = NsDecl(None, "http://relaxng.org/ns/structure/1.0")
_not_allowed = Element.auto(None, _relaxng('notAllowed'))
_start_not_allowed = Element.auto(None, _relaxng('start'), _not_allowed)

def make_ref(name):
    return Element.auto(None, _relaxng('ref'),
                        Attribute(None, 'name', name))

def include_without_start(include):
    new_content = []
    for child in include.content:
        if isinstance(child, Element):
            if child.name == _relaxng('start'):
                new_content.append(_start_not_allowed)
            else:
                new_content.append(child)
        else:
            new_content.append(child)
    return grammar.with_content(new_content)

def grammar_with_new_start(grammar, start):
    new_content = [start]
    for child in grammar.content:
        if isinstance(child, Element):
            if child.name == _relaxng('include'):
                new_content.append(include_without_start(child))
            elif child.name != _relaxng('start'):
                new_content.append(child)
        else:
            new_content.append(child)
    return grammar.with_content(new_content)

def fetch_ctxt(uri):
    """Fetch a relaxng schema from a uri, and return a validation context for it."""
    return libxml2.relaxNGNewParserCtxt(uri).relaxNGParse().relaxNGNewValidCtxt()

def fetch_ctxt_change_start(uri, name):
    """Like fetch_relaxng_ctxt but change the schema's start to one of its definitions.

    Obviously, the schema's top level must be a grammar."""

    schema = funkxml.parse_file(uri)
    grammar = schema.element.orphan
    grammar = grammar_with_new_start(
        grammar,
        Element.auto(None, _relaxng('start'), make_ref(name)))
    doc = None
    try:
        doc = Root([grammar.orphan_with_auto_ns_decls]).to_libxml2()
        rngp = doc.relaxNGNewDocParserCtxt()
        rngs = rngp.relaxNGParse()
        return rngs.relaxNGNewValidCtxt()
    finally:
        if doc != None: doc.freeDoc()
