# Copyright (C) 2004  John Belmonte

"""Time conversion functions"""

import time
import rfc822

ISO_8601_DATE = '%Y-%m-%dZ'
ISO_8601_DATE_AND_TIME = '%Y-%m-%dT%H:%M:%SZ'

_TIME_TUPLE_ISDSDT = 8

def _mkgmtime(gmt_time_tuple):
    """-> seconds since epoch

    Like time.mktime, except it expects GMT instead of local time tuple
    """
    if gmt_time_tuple[_TIME_TUPLE_ISDSDT] == 1:
        offset = time.altzone
    else:
        offset = time.timezone
    return time.mktime(gmt_time_tuple) - offset

def get_date_and_time(seconds=None):
    """Return string for UTC date and time in ISO 8601 "field" format.

    Output example:
        2003-08-25T23:46:14Z
    """
    if seconds is None: seconds = time.time()
    return time.strftime(ISO_8601_DATE_AND_TIME, time.gmtime(seconds))

def get_date(seconds=None):
    """Return string for UTC calendar date in ISO 8601 format.

    Output example:
        2003-08-25Z
    """
    if seconds is None: seconds = time.time()
    return time.strftime(ISO_8601_DATE, time.gmtime(seconds))

def get_seconds(date):
    """-> integer seconds since epoch

    date: ISO 8601 "field" format or calendar date string

    This is the inverse of get_date_and_time() and get_date().
    """
    try:
        time_tuple = time.strptime(date, ISO_8601_DATE_AND_TIME)
    except ValueError:
        time_tuple = time.strptime(date, ISO_8601_DATE)
    # disable DST
    time_tuple = time_tuple[:_TIME_TUPLE_ISDSDT] + (0,)
    return int(_mkgmtime(time_tuple))

def get_http_date(seconds=None):
    """Return string for date and time in RFC 1123 (HTTP-Date) format.

    Output example:
        Tue, 15 Nov 1994 12:45:26 GMT
    """
    return rfc822.formatdate(seconds)

def parse_http_date(date):
    """-> integer seconds since epoch

    date: RFC 1123 (HTTP-Date) format date string
    """
    return int(_mkgmtime(rfc822.parsedate(date)))
